(*======================================================================*
 | unitResourceXPManifests unit for PEResourceExplorer                  |
 |                                                                      |
 | Handle XP manifest resources                                         |
 |                                                                      |
 | The contents of this file are subject to the Mozilla Public License  |
 | Version 1.1 (the "License"); you may not use this file except in     |
 | compliance with the License. You may obtain a copy of the License    |
 | at http://www.mozilla.org/MPL/                                       |
 |                                                                      |
 | Software distributed under the License is distributed on an "AS IS"  |
 | basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See  |
 | the License for the specific language governing rights and           |
 | limitations under the License.                                       |
 |                                                                      |
 | Copyright  Colin Wilson 2002  All Rights Reserved
 |                                                                      |
 | Version  Date        By    Description                               |
 | -------  ----------  ----  ------------------------------------------|
 | 1.0      11/02/2002  CPWW  Original                                  |
 *======================================================================*)

unit unitResourceXPManifests;

interface

uses Windows, Classes, SysUtils, Contnrs, unitResourceDetails, AxCtrls;

type
TXPManifestResourceDetails = class (TResourceDetails)
public
  class function GetBaseType : string; override;
  procedure InitNew; override;
  procedure GetText (xml : TStrings);
  procedure SetText (xml : TStrings);
end;

const
  RT_XPMANIFEST = MakeIntResource (24);

implementation

{ TXPManifestResourceDetails }

const

  manifest : string =
'<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'#13#10+
'<assembly xmlns="urn:schemas-microsoft-com:asm.v1"'#13#10+
'manifestVersion="1.0">'#13#10+
'<assemblyIdentity'#13#10+
'    name="Woozle.PEResourceExplorer.XPManifest"'#13#10+
'    processorArchitecture="x86"'#13#10+
'    version="1.0.0.0"'#13#10+
'    type="win32"/>'#13#10+
'<description>Windows Shell</description>'#13#10+
'<dependency>'#13#10+
'    <dependentAssembly>'#13#10+
'        <assemblyIdentity'#13#10+
'            type="win32"'#13#10+
'            name="Microsoft.Windows.Common-Controls"'#13#10+
'            version="6.0.0.0"'#13#10+
'            processorArchitecture="x86"'#13#10+
'            publicKeyToken="6595b64144ccf1df"'#13#10+
'            language="*"'#13#10+
'        />'#13#10+
'    </dependentAssembly>'#13#10+
'</dependency>'#13#10+
'</assembly>';


class function TXPManifestResourceDetails.GetBaseType: string;
begin
  result := IntToStr (Integer (RT_XPMANIFEST));
end;

procedure TXPManifestResourceDetails.GetText(xml: TStrings);
begin
  data.Seek(0, soFromBeginning);
  xml.LoadFromStream(data);
end;

procedure TXPManifestResourceDetails.InitNew;
begin
  Data.Clear;
  Data.Write(PChar (manifest)^, Length (manifest))
end;

procedure TXPManifestResourceDetails.SetText(xml: TStrings);
begin
  data.Clear;
  xml.SaveToStream(data);
end;

initialization
  RegisterResourceDetails (TXPManifestResourceDetails);
finalization
  UnregisterResourceDetails (TXPManifestResourceDetails);
end.
